float4x4 worldView;
float3x3 worldViewIT;
float3x3 viewInverse;
float4x4 worldViewProjection;

float3 refractiveIndices = {0.980f, 0.970f, 0.960f};

texture cubeMap;
sampler cubeMapSampler = 
sampler_state
{
	Texture = <cubeMap>;
};

struct VSInput
{
	float4 position					: POSITION;
	float3 normal					: NORMAL;
};

struct VSOutput
{
	float4 position					: POSITION;
	float4 fresnel					: COLOR0;
	float3 worldSpaceReflection		: TEXCOORD0;
	float3 worldSpaceRefractionR	: TEXCOORD1;
	float3 worldSpaceRefractionG	: TEXCOORD2;
	float3 worldSpaceRefractionB	: TEXCOORD3;
};

float Fresnel(float3 normal, float3 eyeVector)
{
	float t = 1.0f - dot(normal, eyeVector);
	
	return t * t;
}

VSOutput MetaballVS(VSInput input)
{
	VSOutput output;
	
	output.position			= mul(input.position, worldViewProjection);
	
	float4 eyeSpacePosition = mul(input.position, worldView);
	float3 eyeSpaceEyeVector = normalize(-eyeSpacePosition).xyz;
	
	float3 eyeSpaceNormal = normalize(mul(input.normal, worldViewIT));
	
	output.fresnel = Fresnel(eyeSpaceNormal, eyeSpaceEyeVector);
	
	float3 eyeSpaceReflection = reflect(-eyeSpaceEyeVector, eyeSpaceNormal);
	output.worldSpaceReflection = mul(eyeSpaceReflection, viewInverse);
	
	float3 eyeSpaceRefractionR = refract(-eyeSpaceEyeVector, eyeSpaceNormal, refractiveIndices.r);
	output.worldSpaceRefractionR = mul(eyeSpaceRefractionR, viewInverse);
	
	float3 eyeSpaceRefractionG = refract(-eyeSpaceEyeVector, eyeSpaceNormal, refractiveIndices.g);
	output.worldSpaceRefractionG = mul(eyeSpaceRefractionG, viewInverse);
	
	float3 eyeSpaceRefractionB = refract(-eyeSpaceEyeVector, eyeSpaceNormal, refractiveIndices.b);
	output.worldSpaceRefractionB = mul(eyeSpaceRefractionB, viewInverse);
	
	return output;
}

struct PSInput
{
	float4 fresnel					: COLOR0;
	float3 worldSpaceReflection		: TEXCOORD0;
	float3 worldSpaceRefractionR	: TEXCOORD1;
	float3 worldSpaceRefractionG	: TEXCOORD2;
	float3 worldSpaceRefractionB	: TEXCOORD3;
};

struct PSOutput
{
	float4 color					: COLOR0;
};

PSOutput MetaballPS(PSInput input)
{
	PSOutput output;
	
	float3 reflection = texCUBE(cubeMapSampler, input.worldSpaceReflection);
	
	float3 refraction;
	refraction.r = texCUBE(cubeMapSampler, input.worldSpaceRefractionR).r;
	refraction.g = texCUBE(cubeMapSampler, input.worldSpaceRefractionG).g;
	refraction.b = texCUBE(cubeMapSampler, input.worldSpaceRefractionB).b;
	
	output.color.rgb = lerp(refraction, reflection, input.fresnel);
	output.color.a = 1.0f;
	
	return output;
}

technique MetaballTechnique
{
	pass P0
	{
		VertexShader = compile vs_2_0 MetaballVS();
		PixelShader = compile ps_2_0 MetaballPS();
	}
}
